
#include "TAnalysisNebula.hh"
#include "TAnalysisTools.hh"

#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TRandom.h"
#include "TVector3.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cmath>
#include <vector>

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterNeutronDetector.hh"

ClassImp(TAnalysisNebula)

TAnalysisNebula::TAnalysisNebula()
: TAnalysisInterface(),
  fNumberOfNeutrons(1), fVetoMode(1), fSetThreshold(0), fEnergyTrigger(-1)
{;}

TAnalysisNebula::~TAnalysisNebula()
{;}

int TAnalysisNebula::SetNumberOfNeutrons(Int_t numberOfNeutrons)
{
  fNumberOfNeutrons = numberOfNeutrons;

  return 1;
}

int TAnalysisNebula::SetVetoMode(Int_t vetoMode)
{
  fVetoMode = vetoMode;

  return 1;
}

int TAnalysisNebula::SetThreshold(Double_t neutHardwareThreshold, Double_t vetoHardwareThreshold,
				  Double_t neutSoftwareThreshold, Double_t vetoSoftwareThreshold)
{
  fNeutHardwareThreshold = neutHardwareThreshold;
  fVetoHardwareThreshold = vetoHardwareThreshold;
  fNeutSoftwareThreshold = neutSoftwareThreshold;
  fVetoSoftwareThreshold = vetoSoftwareThreshold;

  fSetThreshold = 1;

  return 1;
}

int TAnalysisNebula::SetEnergyTrigger(Int_t energyTrigger)
{
  fEnergyTrigger = energyTrigger;

  return 1;
}

int TAnalysisNebula::CreateAndAnalyzeExpData()
{
  TTree* tree;
  fFileSim->GetObject(fSimParaGlobal->fTreeName, tree);
  if(!tree){
    std::cerr << " cannot find tree" << std::endl;
    return 0;
  }

  TBranch* branchSimulationDataScintillatorArray = 0;
  if(tree->SetBranchAddress("neutronDetector",&gSimulationDataScintillatorArray,&branchSimulationDataScintillatorArray) < 0) return 0;
  TBranch* branchSimulationDataBeamArray = 0;
  if(tree->SetBranchAddress("beam",&gSimulationDataBeamArray,&branchSimulationDataBeamArray) < 0) return 0;

  if(fNumberOfEntries < 0) fNumberOfEntries = tree->GetEntriesFast();
  for(int i=0; i<fNumberOfEntries; i++) {
    if((i+1)%1000 == 0){
      std::cout << "\r analyzed events:" << (i+1) << std::flush;
    }

    Long64_t entry = tree->LoadTree(i);
    branchSimulationDataScintillatorArray->GetEntry(entry);
    branchSimulationDataBeamArray->GetEntry(entry);

    ProcessOneEvent();
  }
  std::cout << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int TAnalysisNebula::ReadParameters(TFile* file)
{
  std::cout << "reading TAnalysisNebula parameters" << std::endl;

  file->GetObject("parameterGlobal", fSimParaGlobal);
  file->GetObject("parameterNeutronDetector", fSimParaNeut);
  if(!fSimParaGlobal){
    std::cerr << " SimParaGlobal does not exist" << std::endl;
    return 0;
  }else if(!fSimParaNeut){
    std::cerr << " SimParaNeut does not exist" << std::endl;
    return 0;
  }

  fTotalNeutNumber = fSimParaNeut->fNeutNumber*fSimParaNeut->fNeutLayerNumber*fSimParaNeut->fStuckNumber;
  fTotalVetoNumber = fSimParaNeut->fVetoNumber*1*fSimParaNeut->fStuckNumber;
  fTotalModuleNumber = fTotalNeutNumber + fTotalVetoNumber;

#ifdef DEBUG // check consistency with NeutronDetectorConstruction
  TVector3  fNeutPos = TVector3(0, -(fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap)*(fSimParaNeut->fNeutNumber-1)/2, 0);
  TVector3  fNeutPosShift = TVector3(0, fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap, 0);
  TVector3  fVetoPos = TVector3(0,
				-(fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y())*(fSimParaNeut->fVetoNumber-1)/2,
				fSimParaNeut->fVetoGap.z()/2);
  TVector3 fVetoPosShift = TVector3(0, fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y(), fSimParaNeut->fVetoGap.z());

  TVector3 fNeutLayerSize = TVector3(fSimParaNeut->fNeutSize.x(),
				     (fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap)*fSimParaNeut->fNeutNumber-fSimParaNeut->fNeutGap,
				     fSimParaNeut->fNeutSize.z());
  TVector3 fVetoLayerSize = TVector3(fSimParaNeut->fVetoSize.x(),
				     (fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y())*fSimParaNeut->fVetoNumber-fSimParaNeut->fVetoGap.y(),
				     fSimParaNeut->fVetoSize.z()+std::abs(fSimParaNeut->fVetoGap.z()));
  double fNeutStuckSizeZ = (fNeutLayerSize.z()+fSimParaNeut->fNeutLayerGap)*fSimParaNeut->fNeutLayerNumber-fSimParaNeut->fNeutLayerGap;
  double fVetoStuckSizeZ = fVetoLayerSize.z()+fSimParaNeut->fNeutVetoGap;
  TVector3 fNeutLayerPos = TVector3(0, 0, -(fNeutStuckSizeZ-fVetoStuckSizeZ-fSimParaNeut->fNeutSize.z())/2);
  TVector3 fVetoLayerPos = TVector3(0, 0, -fNeutStuckSizeZ/2);
  TVector3 fNeutLayerPosShift = TVector3(0, 0, fNeutLayerSize.z()+fSimParaNeut->fNeutLayerGap);

  double fNeutLongSide = std::max(fNeutLayerSize.x(), fNeutLayerSize.y());
  double fVetoLongSide = std::max(fVetoLayerSize.x(), fVetoLayerSize.y());
  TVector3 fStuckSize = TVector3(std::max(fNeutLongSide, fVetoLongSide),
				 std::max(fNeutLongSide, fVetoLongSide),
				 fNeutStuckSizeZ+fVetoStuckSizeZ);
  TVector3 fStuckPos = TVector3(fSimParaNeut->fPosition.x()+0.01,
				fSimParaNeut->fPosition.y()+0.01,
				fSimParaNeut->fPosition.z()+(fNeutStuckSizeZ-fVetoStuckSizeZ)/2); // neutの面
  TVector3 fStuckPosShift = TVector3(0, 0, fStuckSize.z()+fSimParaNeut->fStuckGap);

  std::cout << 0 << " " << -(fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap)*(fSimParaNeut->fNeutNumber-1)/2 << " " << 0 << " ";
  std::cout << 0 << " " << fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap << " " << 0 << " ";
  std::cout << 0 << " " << -(fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y())*(fSimParaNeut->fVetoNumber-1)/2 << " " << fSimParaNeut->fVetoGap.z()/2 << " ";
  std::cout << 0 << " " << fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y() << " " << fSimParaNeut->fVetoGap.z() << " ";
  std::cout << fSimParaNeut->fNeutSize.x() << " " << (fSimParaNeut->fNeutSize.y()+fSimParaNeut->fNeutGap)*fSimParaNeut->fNeutNumber-fSimParaNeut->fNeutGap << " " << fSimParaNeut->fNeutSize.z() << " ";
  std::cout << fSimParaNeut->fVetoSize.x() << " " << (fSimParaNeut->fVetoSize.y()+fSimParaNeut->fVetoGap.y())*fSimParaNeut->fVetoNumber-fSimParaNeut->fVetoGap.y() << " " << fSimParaNeut->fVetoSize.z()+std::abs(fSimParaNeut->fVetoGap.z()) << " ";
  std::cout << (fNeutLayerSize.z()+fSimParaNeut->fNeutLayerGap)*fSimParaNeut->fNeutLayerNumber-fSimParaNeut->fNeutLayerGap << " ";
  std::cout << fVetoLayerSize.z()+fSimParaNeut->fNeutVetoGap << " ";
  std::cout << 0 << " " << 0 << " " << -(fNeutStuckSizeZ-fVetoStuckSizeZ-fSimParaNeut->fNeutSize.z())/2 << " ";
  std::cout << 0 << " " << 0 << " " << -fNeutStuckSizeZ/2 << " ";
  std::cout << 0 << " " << 0 << " " << fNeutLayerSize.z()+fSimParaNeut->fNeutLayerGap << " ";
  std::cout << std::max(fNeutLayerSize.x(), fNeutLayerSize.y()) << " ";
  std::cout << std::max(fVetoLayerSize.x(), fVetoLayerSize.y()) << " ";
  std::cout << std::max(fNeutLongSide, fVetoLongSide) << " " << std::max(fNeutLongSide, fVetoLongSide) << " " << fNeutStuckSizeZ+fVetoStuckSizeZ << " ";
  //  std::cout << fSimParaNeut->fPosition.x()+0.01 << " " << fSimParaNeut->fPosition.y()+0.01 << " " << fSimParaNeut->fPosition.z()+(fNeutStuckSizeZ-fVetoStuckSizeZ)/2 << " ";
  std::cout << fSimParaNeut->fStuckPos.x() << " " << fSimParaNeut->fStuckPos.y() << " " << fSimParaNeut->fStuckPos.z() << " ";
  std::cout << 0 << " " <<  0 << " " << fStuckSize.z()+fSimParaNeut->fStuckGap << " ";
  std::cout << fSimParaNeut->fSize.x() << " " << fSimParaNeut->fSize.y() << " " << fSimParaNeut->fSize.z() << " ";
  std::cout << fSimParaNeut->fNeutFaceZ << " ";
#endif

  // nebula
  if(fSetThreshold == 0){
    fNeutHardwareThreshold = 2; // MeVee
    fVetoHardwareThreshold = 0.1; // MeVee
    fNeutSoftwareThreshold = 6; // MeVee
    fVetoSoftwareThreshold = 1; // MeVee
  }
  fTimeResolution = 0.2*sqrt(2); // ns
  //  fTimeResolution = 0.175*sqrt(2); // ns
  //  fTimeResolution = 0.1*sqrt(2); // ns
  //  fTimeResolution = 0.01*sqrt(2); // ns
  //  fTimeResolution = 0;

  // mona
  if(fSetThreshold == 0){
    //  fNeutHardwareThreshold = 0.1; // MeVee
    //  fNeutSoftwareThreshold = 0.4; // MeVee
  }
  //  fTimeResolution = 0.3*sqrt(2); // ns

  fNeutAttenuationLength = 668; // cm
  fVetoAttenuationLength = 258; // cm
  fPropagationVelocity = 15.8; // cm/ns

  return 1;
}

int TAnalysisNebula::PrintParameters()
{
  std::cout << "TotalNeutNumber: " << fTotalNeutNumber << "\n"
	    << "TotalVetoNumber: " << fTotalVetoNumber << "\n"
	    << "TotalModuleNumber: " << fTotalModuleNumber << "\n"

	    << "NeutHardwareThreshold: " << fNeutHardwareThreshold << "\n"

	    << "VetoHardwareThreshold: " << fVetoHardwareThreshold << "\n"
	    << "NeutSoftwareThreshold: " << fNeutSoftwareThreshold << "\n"
	    << "VetoSoftwareThreshold: " << fVetoSoftwareThreshold << "\n"
	    << "TimeResolution: " << fTimeResolution << "\n"

	    << "NeutAttenuationLength: " << fNeutAttenuationLength << "\n"
	    << "VetoAttenuationLength: " << fVetoAttenuationLength << "\n"
	    << "PropagationVelocity: " << fPropagationVelocity << "\n"

	    << *fSimParaGlobal << "\n"
	    << *fSimParaNeut << "\n"
    	    << std::flush;

  return 1;
}

int TAnalysisNebula::CreateHist()
{
  fHMulti = new TH1I("hmulti", "multiplicity", 21, -0.5, 20.5);
  fHIDQAveAll = new TH2D("hidqaveall", "id vs qave all", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);

//  fHIDX = new TH2D("hidx", "id vs x", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, -fSimParaNeut->fSize.x()/2-10, fSimParaNeut->fSize.x()/2+10);
//  fHIDY = new TH2D("hidy", "id vs y", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, -fSimParaNeut->fSize.y()/2-10, fSimParaNeut->fSize.y()/2+10);
//  fHIDZ = new TH2D("hidz", "id vs z", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5,1000,
//		   fSimParaNeut->fPosition.z()-100, fSimParaNeut->fPosition.z()+fSimParaNeut->fSize.z()+100);
  fHIDX = new TH2D("hidx", "id vs x", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 400, -200, 200);
  fHIDY = new TH2D("hidy", "id vs y", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 400, -200, 200);
  fHIDZ = new TH2D("hidz", "id vs z", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 400, 1000, 1400);

  fHIDQL = new TH2D("hidql", "id vs qu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDQR = new TH2D("hidqr", "id vs qd", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTL = new TH2D("hidtl", "id vs tu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 200);
  fHIDTR = new TH2D("hidtr", "id vs td", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 200);
  fHIDQAve = new TH2D("hidqave", "id vs qave", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTAve = new TH2D("hidtave", "id vs tave", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 200);

  fhenergy = new TH1D("henergy", "neutron energy", 600, 0, 300);
  fhenergytheta = new TH2D("henergytheta", "neutron energy vs theta", 6000, 0, 300, 200, 0, 0.1);
  fhidenergy = new TH2D("hidenergy", "id vs neutron energy", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 600, 0, 300);
  fhenergyqave = new TH2D("henergyqave", "neutron energy vs qave", 600, 0, 300, 300, 0, 150);

  fHXTheta = new TH2D("hxtheta", "x vs theta",  300, -fSimParaNeut->fSize.x()/2-10, fSimParaNeut->fSize.x()/2+10, 200, 0, 0.1);
  fHTanXTanY = new TH2D("htanxtany", "tanx vs tany", 300, -0.1, 0.1, 300, -0.1, 0.1);
  //  fHTanXTanY2 = new TH2D("htanxtany2", "tanx vs tany 2", 300, -0.1, 0.1, 300, -0.1, 0.1);
  //  fHTanXTanY3 = new TH2D("htanxtany3", "tanx vs tany 3", 300, -0.1, 0.1, 300, -0.1, 0.1);
  fHBeamTanXBeamTanY = new TH2D("hbeamtanxbeamtany", "beam tanx vs beamtany", 300, -0.1, 0.1, 300, -0.1, 0.1);
  fHBeamTanXTanX = new TH2D("hbeamtanxtanx", "beam tanx vs tanx", 300, -0.1, 0.1, 300, -0.1, 0.1);
  fHBeamTanYTanY = new TH2D("hbeamtanytany", "beam tany vs tany", 300, -0.1, 0.1, 300, -0.1, 0.1);
  fHDTanXDTanY = new TH2D("hdtanxdtany", "dtanx vs dtany", 300, -0.1, 0.1, 300, -0.1, 0.1);
  fHEnergyContami = new TH1D("henergycontami", "energy contaminate", 600, 0, 300);
  //  fHEnergyContami2 = new TH1D("henergycontami2", "energy contaminate 2", 600, 0, 300);
  //  fHEnergyContami3 = new TH1D("henergycontami3", "energy contaminate 3", 600, 0, 300);
  fHBPPX = new TH2D("hbppx", "beam p vs p x", 300, -100, 100, 300, -100, 100);  
  fHBPPY = new TH2D("hbppy", "beam p vs p y", 300, -100, 100, 300, -100, 100);  
  fHBPPZ = new TH2D("hbppz", "beam p vs p z", 300,    0,1000, 300,    0,1000);  
  fHDPX = new TH1D("hdpx", "beam p - p x", 300, -0.02, 0.02);
  fHDPY = new TH1D("hdpy", "beam p - p y", 300, -0.02, 0.02);
  fHDPZ = new TH1D("hdpz", "beam p - p z", 300, -0.02, 0.02);
  fHDX = new TH1D("hdx", "beam x - x", 300, -30, 30);
  fHDY = new TH1D("hdy", "beam x - y", 300, -30, 30);
  fHDT = new TH1D("hdt", "beam t - t", 300, -1, 1);

  if(fNumberOfNeutrons == 2){
    fhenergy1n = new TH1D("henergy1n", "neutron energy 1n", 600, 0, 300);

    fhq1q2       = new TH2D("hq1q2", "q1 vs q2", 600, 0, 150, 600, 0, 150);

    fhdtq1      = new TH2D("hdtq1", "2n dt vs q1", 400, -40, 40, 600, 0, 150);
    fhdtq2      = new TH2D("hdtq2", "2n dt vs q2", 400, -40, 40, 600, 0, 150);
    fhdbetaq1   = new TH2D("hdbetaq1", "2n delta beta vs q1", 400, -2, 2, 600, 0, 150);
    fhdbetaq2   = new TH2D("hdbetaq2", "2n delta beta vs q2", 400, -2, 2, 600, 0, 150);
    fhivdbetaq1 = new TH2D("hivdbetaq1", "2n 1/(delta beta) vs q1", 400, -15, 15, 600, 0, 150);
    fhivdbetaq2 = new TH2D("hivdbetaq2", "2n 1/(delta beta) vs q2", 400, -15, 15, 600, 0, 150);
    fhbetaddbetaq1   = new TH2D("hbetaddbetaq1", "2n delta beta/beta vs q1", 400, -2, 2, 600, 0, 150);
    fhbetaddbetaq2   = new TH2D("hbetaddbetaq2", "2n delta beta/beta vs q2", 400, -2, 2, 600, 0, 150);
    fhivbetaddbetaq1   = new TH2D("ivhbetaddbetaq1", "2n 1/(delta beta/beta) vs q1", 400, -10, 10, 600, 0, 150);
    fhivbetaddbetaq2   = new TH2D("ivhbetaddbetaq2", "2n 1/(delta beta/beta) vs q2", 400, -10, 10, 600, 0, 150);
  }else if(fNumberOfNeutrons == 12){
    fhd12 = new TH1D("hd12", "d12", 30, 0, 300);
    fhtheta12 = new TH1D("htheta12", "theta12", 30, 0, 180);
    fhv12 = new TH1D("hv12", "v12", 30, 0, 20);
  }

  fHTQ.resize(fTotalModuleNumber, (TH1*)0);
  fHQT.resize(fTotalModuleNumber, (TH1*)0);
  fHXY.resize(fTotalModuleNumber, (TH1*)0);
  //    for(int i=0; i<fTotalModuleNumber+1; ++i){
  for(int i=0; i<1; ++i){
    {
      std::ostringstream name,title;
      name << "htq" << i;
      title << "t vs q id" << i;
      fHTQ[i] = new TH2D(name.str().c_str(), title.str().c_str(), 1000,-5,5, 300,0,150);
    }
    {
      std::ostringstream name,title;
      name << "hqt" << i;
      title << "q vs t id" << i;
      fHQT[i] = new TH2D(name.str().c_str(), title.str().c_str(), 300,0,150, 1000,-5,5);
    }
  }
  for(int i=0; i<1; ++i){
    {
      std::ostringstream name,title;
      name << "hxy" << i;
      title << "x vs y id" << i;
      fHXY[i] = new TH2D(name.str().c_str(), title.str().c_str(), 400,-100,100, 400,-100,100);
    }
  }

  return 1;
}

int TAnalysisNebula::ProcessOneEvent()
{
  if(gSimulationDataScintillatorArray->empty()){
#ifdef DEBUG
    std::cout << " no events" << std::endl;
#endif
    return 0;
  }

  // --------------------- create experiment data ---------------------
  fID.clear();
  fMT.clear();
  fQL.clear();
  fQR.clear();
  fTL.clear();
  fTR.clear();
  SimToExp();

  // ---------------------- fill data container --------------------------
  std::vector<TAnalysisNebulaData> data_v;
  for(int j=0; j<(int)fID.size(); ++j){
    TAnalysisNebulaData data;
    std::vector<int> id = fID[j];
    data.id = fID[j];
    data.sid = -1;
    if(fMT[j] == kNEUT){
      data.sid = 1+id[0]+id[1]*fSimParaNeut->fNeutNumber+id[2]*fSimParaNeut->fNeutNumber*fSimParaNeut->fNeutLayerNumber;
    }else if(fMT[j] == kVETO){
      data.sid = 1+id[0]+id[2]*fSimParaNeut->fVetoNumber+fTotalNeutNumber;
    }
    data.mt = fMT[j];
    data.qu = fQL[j];
    data.qd = fQR[j];
    data.tu = fTL[j];
    data.td = fTR[j];
    if(data.mt == kNEUT){
      data.qave = sqrt(data.qu*data.qd)*exp(fSimParaNeut->fNeutSize.X()/(2*fNeutAttenuationLength));
      data.tave = (data.tu+data.td)/2-fSimParaNeut->fNeutSize.X()/fPropagationVelocity/2;
    }else{
      data.qave = sqrt(data.qu*data.qd)*exp(fSimParaNeut->fVetoSize.X()/(2*fVetoAttenuationLength));
      data.tave = (data.tu+data.td)/2-fSimParaNeut->fVetoSize.X()/fPropagationVelocity/2;
    }	
    data.dt = -(data.tu-data.td);

    if(data.mt == kNEUT){
      data.pos = fSimParaNeut->fNeutPos + fSimParaNeut->fNeutPosShift*id[0];
      data.pos += fSimParaNeut->fNeutLayerPos + fSimParaNeut->fNeutLayerPosShift*id[1];
    }else{
      data.pos.SetX(fSimParaNeut->fVetoPosShift.x()*id[0]);
      data.pos.SetY(fSimParaNeut->fVetoPosShift.y()*id[0]);
      if(id[0]%2){
	data.pos.SetZ(-1*fSimParaNeut->fVetoPosShift.z());
      }
      data.pos += fSimParaNeut->fVetoPos;
      data.pos += fSimParaNeut->fVetoLayerPos;
    }
    data.pos += fSimParaNeut->fStuckPos + fSimParaNeut->fStuckPosShift*id[2];
    data.pos.SetX(data.pos.x()+0.01+data.dt*fPropagationVelocity/2);
    data.pos.SetY(data.pos.y()+0.01);
    data.pos.SetZ(data.pos.z()-fSimParaNeut->fNeutFaceZ);
    //    data.pos.SetY(data.pos.y()+0.01+gRandom->Uniform(fSimParaNeut->fNeutSize.Y())-fSimParaNeut->fNeutSize.Y()/2);
    //    data.pos.SetZ(data.pos.z()-fSimParaNeut->fNeutFaceZ+gRandom->Uniform(fSimParaNeut->fNeutSize.Z())-fSimParaNeut->fNeutSize.Z()/2);

    if(fSimParaNeut->fNeutRotationMode == 1){
      double y = data.pos.x();
      double x = data.pos.y();
      data.pos.SetX(x);
      data.pos.SetY(y);
    }else if(fSimParaNeut->fNeutRotationMode == 2){
      if(id[1]%2){
	double y = data.pos.x();
	double x = data.pos.y();
	data.pos.SetX(x);
	data.pos.SetY(y);
      }
    }
     
    // add overall offset
    data.pos += fSimParaNeut->fPosition;

    // to adjust to experiment
    data.pos.SetX(-data.pos.x());

    //    std::cout << data.pos.x() << " " << data.pos.y() << " " << data.pos.z() << " " << fSimParaNeut->fPosition.z() << std::endl;

    Double_t time = data.tave;
    TVector3 beta, p;
    for(Int_t i=0; i<3; ++i){
      beta[i] = data.pos[i]/time/vlight;
    }
    Double_t betaabs = sqrt(beta[0]*beta[0]+beta[1]*beta[1]+beta[2]*beta[2]);
    Double_t gamma = 1/sqrt(1-betaabs*betaabs);
    for(Int_t i=0; i<3; ++i){
      p[i] = mneutron*gamma*beta[i];
    }
    data.time = time;
    data.betaabs = betaabs;
    data.pabs = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
    data.energy = sqrt(mneutron*mneutron + data.pabs*data.pabs) - mneutron;
    data.theta = atan(sqrt(p[0]*p[0]+p[1]*p[2])/p[2]);
    data.phi = atan(p[1]/p[0]);

    data.beta = beta;
    data.gamma = gamma;
    data.p = p;

    //    std::cout << j << " " << data.sid << " " << data.time << " " << data.pos[0] << " " << data.pos[1] << " " << data.pos[2] << " " << time << " " << vlight << " " << beta[0] << " " << beta[1] << " " << beta[2] << " " << data.betaabs << " " << data.pabs << " " << data.energy << " " << data.theta << std::endl;

    data_v.push_back(data);
  }

  if(data_v.empty()) return 1;

  //  std::cout << gSimulationDataBeamArray->size() << std::endl;
  TSimulationDataBeam dataBeam = (*gSimulationDataBeamArray)[0];
  TVector3 beammomentum = dataBeam.fMomentum;
  double beamp = dataBeam.fMomentum.Mag();
  double beamm = dataBeam.fMass;
  double beammovp = beamm/beamp;
  double beambeta = 1/sqrt(beammovp*beammovp+1);
  double beamenergy = sqrt(beamm*beamm+beamp*beamp);
  double beamk = beamenergy - beamm;
//  std::cout << beamp << " "
//	    << beamm << " "
//	    << beambeta << " "
//	    << beamenergy << " "
//	    << beamk << " "
//	    << std::endl;

// --- Erel ---
//  TVector3 ptot;
//  Double_t etot=0;
//  Double_t mtot=0;
//  for(int i=0; i<5; ++i){
//    TSimulationDataBeam dataBeam = (*gSimulationDataBeamArray)[i];
//    ptot += dataBeam.fMomentum;
//    etot += sqrt(pow(dataBeam.fMomentum.Mag(),2)+pow(dataBeam.fMass,2));
//    mtot += dataBeam.fMass;
//  }
//  Double_t erel = sqrt(etot*etot - ptot*ptot)-mtot;

  // -------------------------- analyze data -------------------------------------
  TAnalysisNebulaTools::threshold(data_v, fNeutSoftwareThreshold, fVetoSoftwareThreshold);
  if(data_v.empty()) return 1;

  if(fNumberOfNeutrons == 1){
    if(TAnalysisNebulaTools::veto(data_v, fVetoMode)) return 1;
    TAnalysisNebulaTools::killveto(data_v);
    if(data_v.empty()) return 1;

    if(fEnergyTrigger > 0){
      std::vector<TAnalysisNebulaData> data_v_trigger = data_v;    
      TAnalysisNebulaTools::hitmintime(data_v_trigger);
      {
	double emean = -1;
	double esigma = -1;
	if(fEnergyTrigger == 200){
	  emean = 194;
	  esigma = 2.27319;
	}else if(fEnergyTrigger == 250){
	  emean = 246;
	  esigma = 3.22752;
	}
	double energy = data_v_trigger[0].energy;
	//	if(!(energy > emean && energy < emean+esigma*3)) return 1;
	if(!(energy > emean-esigma*3 && energy < emean+esigma*3)) return 1;
      }
      TAnalysisNebulaTools::acceptance(data_v_trigger, 0.04);
      //      TAnalysisNebulaTools::acceptance(data_v_trigger, 0.06);
      if(data_v_trigger.empty()) return 1;
    }

    fHMulti->Fill(data_v.size());
    for(int j=0; j<(int)data_v.size(); ++j){
      fHIDQAveAll->Fill(data_v[j].sid, data_v[j].qave);
    }
    TAnalysisNebulaTools::hitmintime(data_v);
    if(fEnergyTrigger <= 0){
      TAnalysisNebulaTools::acceptance(data_v, 0.04);
    }
  }else if(fNumberOfNeutrons == 2){ // search cross talk
    // select proper 1n hit in first layer
    std::vector<TAnalysisNebulaData> data_v_copy = data_v;
    if(TAnalysisNebulaTools::veto(data_v_copy, 1)) return 1;
    TAnalysisNebulaTools::killveto(data_v_copy);
    TAnalysisNebulaTools::hitmintime(data_v_copy);
    //      TAnalysisNebulaTools::acceptance(data_v_copy, 0.04);
    if(data_v_copy.empty()) return 1;
    if(!(data_v_copy[0].energy > beamk - 5 && data_v_copy[0].energy < beamk + 5)) return 1;
    fhenergy1n->Fill(data_v_copy[0].energy);

    // select 2hit
    if(TAnalysisNebulaTools::veto(data_v, fVetoMode)) return 1;
    TAnalysisNebulaTools::killveto(data_v);
    fHMulti->Fill(data_v.size());
    for(int j=0; j<(int)data_v.size(); ++j){
      fHIDQAveAll->Fill(data_v[j].sid, data_v[j].qave);
    }
    TAnalysisNebulaTools::hitmintime2(data_v);
    if(data_v.size() != 2) return 1;
  }else if(fNumberOfNeutrons == 12){ // search 2 earliest event
    // select 2hit
    if(TAnalysisNebulaTools::veto(data_v, fVetoMode)) return 1;
    TAnalysisNebulaTools::killveto(data_v);
    if(data_v.empty()) return 1;
    fHMulti->Fill(data_v.size());
    for(int j=0; j<(int)data_v.size(); ++j){
      fHIDQAveAll->Fill(data_v[j].sid, data_v[j].qave);
    }
    if(data_v.size() < 2) return 1;
    TAnalysisNebulaTools::hitmintime12(data_v);
    if(data_v.size() != 2) return 1;
  }   

  // cross talk fill
  if(fNumberOfNeutrons == 2){
    TAnalysisNebulaData n1 = data_v[0];
    TAnalysisNebulaData n2 = data_v[1];
    if(data_v[0].pos[2] > data_v[1].pos[2]){
      n1 = data_v[1];
      n2 = data_v[0];
    }

    double t1 = n1.tave;
    double t2 = n2.tave;
    double q1 = n1.qave;
    double q2 = n2.qave;
    TVector3 pos1 = n1.pos;
    TVector3 pos2 = n2.pos;

    double x1 = pos1[0];
    double y1 = pos1[1];
    double z1 = pos1[2];
    double x2 = pos2[0];
    double y2 = pos2[1];
    double z2 = pos2[2];
    double dx = x2-x1;
    double dy = y2-y1;
    double dz = z2-z1;
    double dr = sqrt(dx*dx+dy*dy+dz*dz);
    double dt = t2-t1;
    double dbeta = dr/dt/vlight;
    double betaddbeta = (t1 < t2) ? dbeta/n1.betaabs : dbeta/n2.betaabs;
    double ivbetaddbeta = 1/betaddbeta;

    fhq1q2->Fill(q1,q2);

    fhdtq1->Fill(dt, q1);
    fhdtq2->Fill(dt, q2);
    fhdbetaq1->Fill(dbeta, q1);
    fhdbetaq2->Fill(dbeta, q2);
    fhivdbetaq1->Fill(1/dbeta, q1);
    fhivdbetaq2->Fill(1/dbeta, q2);
    fhbetaddbetaq1->Fill(betaddbeta, q1);
    fhbetaddbetaq2->Fill(betaddbeta, q2);
    fhivbetaddbetaq1->Fill(ivbetaddbeta, q1);
    fhivbetaddbetaq2->Fill(ivbetaddbeta, q2);
  }else if(fNumberOfNeutrons == 12){
    TAnalysisNebulaData n1 = data_v[0];
    TAnalysisNebulaData n2 = data_v[1];
    if(data_v[0].tave > data_v[1].tave){
      n1 = data_v[1];
      n2 = data_v[0];
    }

    double t1 = n1.tave;
    double t2 = n2.tave;
    double q1 = n1.qave;
    double q2 = n2.qave;
    TVector3 pos1 = n1.pos;
    TVector3 pos2 = n2.pos;

    double x1 = pos1[0];
    double y1 = pos1[1];
    double z1 = pos1[2];
    //    double y1 = pos1[1]+gRandom->Uniform(fSimParaNeut->fNeutSize.Y())-fSimParaNeut->fNeutSize.Y()/2;
    //    double z1 = pos1[2]+gRandom->Uniform(fSimParaNeut->fNeutSize.Z())-fSimParaNeut->fNeutSize.Z()/2;
    double x2 = pos2[0];
    double y2 = pos2[1];
    double z2 = pos2[2];
    //    double y2 = pos2[1]+gRandom->Uniform(fSimParaNeut->fNeutSize.Y())-fSimParaNeut->fNeutSize.Y()/2;
    //    double z2 = pos2[2]+gRandom->Uniform(fSimParaNeut->fNeutSize.Z())-fSimParaNeut->fNeutSize.Z()/2;
    double dx = x2-x1;
    double dy = y2-y1;
    double dz = z2-z1;
    double dr = sqrt(dx*dx+dy*dy+dz*dz);
    double dtheta = atan(sqrt(dx*dx+dy*dy)/dz);
    if(dtheta < 0) dtheta += M_PI;
    double dt = t2-t1;
    double dv = dr/dt;

    fhd12->Fill(dr);
    fhtheta12->Fill(dtheta/M_PI*180);
    fhv12->Fill(dv);
  }

  // 1n fill
  for(int j=0; j<(int)data_v.size(); ++j){
    Int_t id = data_v[j].sid;
    Double_t qu = data_v[j].qu;
    Double_t qd = data_v[j].qd;
    Double_t tu = data_v[j].tu;
    Double_t td = data_v[j].td;
    Double_t qave = data_v[j].qave;
    Double_t tave = data_v[j].tave;
    TVector3 pos = data_v[j].pos;

    fHIDX->Fill(id, pos.x());
    fHIDY->Fill(id, pos.y());
    fHIDZ->Fill(id, pos.z());
    fHIDQL->Fill(id, qu);
    fHIDQR->Fill(id, qd);
    fHIDTL->Fill(id, tu);
    fHIDTR->Fill(id, td);
    fHIDQAve->Fill(id, qave);
    fHIDTAve->Fill(id, tave);

    double energy = data_v[j].energy;
    double posx = pos[0]+ gRandom->Uniform(-6,6);
    double theta = atan(sqrt(posx*posx+pos[1]*pos[1])/pos[2]);
    fhenergy->Fill(energy);
    fhenergytheta->Fill(energy,theta);
    fhidenergy->Fill(id, energy);
    fhenergyqave->Fill(energy,qave);

    fHXTheta->Fill(posx, theta);

    double beamtanx = beammomentum.x()/beammomentum.z();
    double beamtany = beammomentum.y()/beammomentum.z();
    double tanx = posx/pos.z();
    double tany = pos.y()/pos.z();
    fHTanXTanY->Fill(tanx, tany);
    fHBeamTanXBeamTanY->Fill(-beamtanx, beamtany);
    fHBeamTanXTanX->Fill(-beamtanx, tanx);
    fHBeamTanYTanY->Fill(beamtany, tany);
    fHDTanXDTanY->Fill(-beamtanx-tanx, beamtany-tany);
    fHEnergyContami->Fill(energy);

    double r = sqrt(pos[0]*pos[0]+pos[1]*pos[1]+pos[2]*pos[2]);
    double TOF = r/beambeta/29.979;
    for(int i=0; i<1; ++i){
      fHTQ[i]->Fill(tave - TOF, qave);
      fHQT[i]->Fill(qave, tave - TOF);
    }

    fHXY[0]->Fill(pos[0],pos[1]);

    //    std::cout << beammomentum[0] << " " << data_v[j].p[0] << std::endl;
    fHBPPX->Fill(beammomentum[0],data_v[j].p[0]);
    fHBPPY->Fill(beammomentum[1],data_v[j].p[1]);
    fHBPPZ->Fill(beammomentum[2],data_v[j].p[2]);
    fHDPX->Fill((beammomentum[0]+data_v[j].p[0])/beamp);
    fHDPY->Fill((beammomentum[1]-data_v[j].p[1])/beamp);
    fHDPZ->Fill((beammomentum[2]-data_v[j].p[2])/beamp);
    fHDX->Fill(beamtanx*pos[2]+pos[0]);
    fHDY->Fill(beamtany*pos[2]-pos[1]);
    fHDT->Fill(data_v[j].pos[2]/(29.979*beambeta*beammomentum[2]/beamp)-data_v[j].time);
  }

  return 1;
}
  
void TAnalysisNebula::SimToExp()
{
  for(int i=0; i<(int)gSimulationDataScintillatorArray->size(); ++i){
    TSimulationDataScintillator simulationDataScintillator = (*gSimulationDataScintillatorArray)[i];
 //    std::cout << simulationDataScintillator << std::endl;

    std::vector<Int_t> id = simulationDataScintillator.fModuleID;
    EModuleType mt;
    if(simulationDataScintillator.fModuleName == "Neut"){ 
      mt = kNEUT;
    }else if(simulationDataScintillator.fModuleName == "Veto"){
      mt = kVETO;
    }else{
      //      std::cerr << " error at line " << __LINE__ << std::endl;
    }

    Double_t moduleSizeX = 0;
    Int_t rotationMode = 0;
    Double_t attenuationLength = 0;
    if(mt == kNEUT){
      moduleSizeX = fSimParaNeut->fNeutSize.X();
      rotationMode = fSimParaNeut->fNeutRotationMode;
      attenuationLength = fNeutAttenuationLength;
    }else{
      moduleSizeX = fSimParaNeut->fVetoSize.X();
      rotationMode = fSimParaNeut->fVetoRotationMode;
      attenuationLength = fVetoAttenuationLength;
    }

    Double_t xl = moduleSizeX/2.
      -(simulationDataScintillator.fPrePosition.X()-fSimParaNeut->fPosition.X());
    Double_t xr = moduleSizeX/2.
      +(simulationDataScintillator.fPrePosition.X()-fSimParaNeut->fPosition.X());
    Double_t yl = moduleSizeX/2.
      -(simulationDataScintillator.fPrePosition.Y()-fSimParaNeut->fPosition.Y()); // left == up
    Double_t yr = moduleSizeX/2.
      +(simulationDataScintillator.fPrePosition.Y()-fSimParaNeut->fPosition.Y()); // right == down

    // don't use fPostKineticEnergy (after interaction, it may be '0' !!)
    Double_t energyDepositEE = TAnalysisTools::MeVtoMeVee(simulationDataScintillator.fPreKineticEnergy,
							  simulationDataScintillator.fPreKineticEnergy-
							  simulationDataScintillator.fEnergyDeposit,
							  simulationDataScintillator.fParticleName.Data(),
							  simulationDataScintillator.fPDGCode);

    Double_t tl, tr, ql, qr;
    if(rotationMode == 1){ // stuck horizontal
      tl = simulationDataScintillator.fPreTime + yl/fPropagationVelocity;
      tr = simulationDataScintillator.fPreTime + yr/fPropagationVelocity;
      ql = energyDepositEE*exp(-yl/attenuationLength);
      qr = energyDepositEE*exp(-yr/attenuationLength);
    }else if(rotationMode == 2){ // vertical, horizontal, vertical, ....
      if(id[0]%2){
	tl = simulationDataScintillator.fPreTime + yl/fPropagationVelocity;
	tr = simulationDataScintillator.fPreTime + yr/fPropagationVelocity;
	ql = energyDepositEE*exp(-yl/attenuationLength);
	qr = energyDepositEE*exp(-yr/attenuationLength);
      }else{
	tl = simulationDataScintillator.fPreTime + xl/fPropagationVelocity;
	tr = simulationDataScintillator.fPreTime + xr/fPropagationVelocity;
	ql = energyDepositEE*exp(-xl/attenuationLength);
	qr = energyDepositEE*exp(-xr/attenuationLength);
      }
    }else{ // stuck vertical
      tl = simulationDataScintillator.fPreTime + xl/fPropagationVelocity;
      tr = simulationDataScintillator.fPreTime + xr/fPropagationVelocity;
      ql = energyDepositEE*exp(-xl/attenuationLength);
      qr = energyDepositEE*exp(-xr/attenuationLength);
    }      

    bool flagFound = false;
    for(int j=0; j<(int)fID.size(); ++j){
      if(fID[j] == id){
	flagFound = true;
	if(fTL[j] > tl) fTL[j] = tl;
	if(fTR[j] > tr) fTR[j] = tr;
	fQL[j] += ql;
	fQR[j] += qr;
	break;
      }
    }
    if(!flagFound){
      fID.push_back(id);
      fMT.push_back(mt);
      fTL.push_back(tl);
      fTR.push_back(tr);
      fQL.push_back(ql);
      fQR.push_back(qr);
    }
  }

  // hardware threshold
  for(int i=0; i<(int)fID.size(); ++i){
    Double_t threshold;
    if(fMT[i] == kNEUT){
      threshold = fNeutHardwareThreshold;
    }else{
      threshold = fVetoHardwareThreshold;
    }
    if(fQL[i] < threshold || fQR[i] < threshold){
      fID.erase(fID.begin()+i);
      fMT.erase(fMT.begin()+i);
      fTL.erase(fTL.begin()+i);
      fTR.erase(fTR.begin()+i);
      fQL.erase(fQL.begin()+i);
      fQR.erase(fQR.begin()+i);
      --i;
      continue;
    }
  }

  // time blur
  for(int i=0; i<(int)fID.size(); ++i){
    fTL[i] += gRandom->Gaus(0,fTimeResolution);
    fTR[i] += gRandom->Gaus(0,fTimeResolution);  
  }
}

// -------- TAnalysisNebulaTools implimentation -------------- //

ClassImp(TAnalysisNebulaTools)

void TAnalysisNebulaTools::threshold(std::vector<TAnalysisNebulaData>& data_v, double neutSoftwareThreshold, double vetoSoftwareThreshold)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    if(data_v[j].mt == kNEUT){
      if(data_v[j].qave < neutSoftwareThreshold){
	data_v.erase(data_v.begin()+j);
	--j;
	continue;
      }
    }else{
      if(data_v[j].qave < vetoSoftwareThreshold){
	data_v.erase(data_v.begin()+j);
	--j;
	continue;
      }
    }
  }
}

void TAnalysisNebulaTools::acceptance(std::vector<TAnalysisNebulaData>& data_v, double theta)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    TAnalysisNebulaData data = data_v[j];
    double posxrand = data.pos[0]+ gRandom->Uniform(-6,6);
    double posy = data.pos[1];
    double posz = data.pos[2];
    double data_theta = atan(sqrt(posxrand*posxrand+posy*posy)/posz);
    if(!(data_theta < theta)){
      data_v.erase(data_v.begin()+j);
      --j;
      continue;
    }
  }
}

bool TAnalysisNebulaTools::veto(std::vector<TAnalysisNebulaData>& data_v, int mode)
{
  bool vetoflag = false;
  for(int j=0; j<(int)data_v.size(); ++j){
    std::vector<int> id = data_v[j].id;
    if(data_v[j].mt == kVETO){
      if(mode == 1){
	if(id[2] == 0){
	  vetoflag = true;
	  break;
	}
      }else if(mode == 2){
	if(id[2] == 0 || id[2] == 1){
	  vetoflag = true;
	  break;
	}
      }
    }
  }

  return vetoflag;
}

void TAnalysisNebulaTools::killveto(std::vector<TAnalysisNebulaData>& data_v)
{
  for(int j=0; j<(int)data_v.size(); ++j){
    if(data_v[j].mt == kVETO){
      data_v.erase(data_v.begin()+j);
      --j;
    }
  }
}

void TAnalysisNebulaTools::hitmintime(std::vector<TAnalysisNebulaData>& data_v)
{
  if(data_v.size() != 0){
    Double_t tmin = data_v[0].tave;
    while((int)data_v.size() > 1){
      Double_t t = data_v[1].tave;

      if(tmin > t){
	tmin = t;
	data_v.erase(data_v.begin());
      }else{
	data_v.erase(data_v.begin()+1);
      }
    }
  }
}

void TAnalysisNebulaTools::hitmintime2(std::vector<TAnalysisNebulaData>& data_v)
{
  if(data_v.size() != 0){
    std::vector<std::vector<TAnalysisNebulaData> > data_v_layer(2);
    for(int j=0; j<(int)data_v.size(); ++j){
      std::vector<int> id = data_v[j].id;
      if(id[2] == 0){ // 1st stuck neut
	data_v_layer[0].push_back(data_v[j]);
      }else if(id[2] == 1){ // 2nd stuck neut
	data_v_layer[1].push_back(data_v[j]);
      }
    }

    if(data_v_layer[0].size() != 0){
      Double_t tmin = data_v_layer[0][0].tave;
      while((int)data_v_layer[0].size() > 1){
	Double_t t = data_v_layer[0][1].tave;

	if(tmin > t){
	  tmin = t;
	  data_v_layer[0].erase(data_v_layer[0].begin());
	}else{
	  data_v_layer[0].erase(data_v_layer[0].begin()+1);
	}
      }
    }

    if(data_v_layer[1].size() != 0){
      Double_t tmin = data_v_layer[1][0].tave;
      while((int)data_v_layer[1].size() > 1){
	Double_t t = data_v_layer[1][1].tave;

	if(tmin > t){
	  tmin = t;
	  data_v_layer[1].erase(data_v_layer[1].begin());
	}else{
	  data_v_layer[1].erase(data_v_layer[1].begin()+1);
	}
      }
    }

    data_v.clear();
    if(data_v_layer[0].size() == 1){
      data_v.push_back(data_v_layer[0][0]);
    }
    if(data_v_layer[1].size() == 1){
      data_v.push_back(data_v_layer[1][0]);
    }
  }
}

void TAnalysisNebulaTools::hitmintime12(std::vector<TAnalysisNebulaData>& data_v)
{
//  std::cout << " aaaa " << std::endl;
//  for(int j=0; j<(int)data_v.size(); ++j){
//    std::cout << data_v[j].tave << " ";
//  }
//  std::cout << std::endl;

  if(data_v.size() > 1){
    Double_t tmin1 = data_v[0].tave;
    Double_t tmin2 = data_v[1].tave;
    while((int)data_v.size() > 2){
      Double_t t = data_v[2].tave;

      if(tmin1 > t && tmin2 > t){
	if(tmin1 < tmin2){
	  tmin2 = t;
	  data_v.erase(data_v.begin()+1);
	}else{
	  tmin1 = tmin2;
	  tmin2 = t;
	  data_v.erase(data_v.begin());
	}
      }else{
	if(tmin1 > t){
	  tmin1 = tmin2;
	  tmin2 = t;
	  data_v.erase(data_v.begin());
	}else if(tmin2 > t){
	  tmin2 = t;
	  data_v.erase(data_v.begin()+1);
	}else{
	  data_v.erase(data_v.begin()+2);
	}
      }
    }
  }

//  for(int j=0; j<(int)data_v.size(); ++j){
//    std::cout << data_v[j].tave << " ";
//  }
//  std::cout << std::endl;
}

